# SPDX-FileCopyrightText: 2025 adamws <adamws@users.noreply.github.com>
#
# SPDX-License-Identifier: GPL-3.0-or-later

import os
from typing import Optional

import wx


class MessageDialog(wx.Dialog):
    def __init__(self, parent: Optional[wx.Window], title: str = "") -> None:
        super(MessageDialog, self).__init__(
            parent, -1, title, style=wx.DEFAULT_DIALOG_STYLE | wx.STAY_ON_TOP
        )
        self.parent = parent

    def adjust_size_to_text(self, text_ctrl: wx.TextCtrl, text: str) -> None:
        dc = wx.ClientDC(self)
        max_line_width = 0

        for line in text.split("\n"):
            width, _ = dc.GetTextExtent(line)
            max_line_width = max(max_line_width, width)

        width_for_text = max_line_width + 50

        if self.parent is None:
            max_width = wx.GetDisplaySize().GetWidth() - 100
        else:
            max_width = self.parent.GetSize().GetWidth() - 100

        final_width = min(width_for_text, max_width)

        text_ctrl.SetMinSize(wx.Size(final_width, -1))
        self.SetMinSize(wx.Size(final_width + 20, -1))


def show_with_test_support(class_or_func, *args, **kwargs) -> wx.Dialog:
    """This exist only for testing purposes"""
    import threading

    app = wx.App()
    dlg = class_or_func(*args, **kwargs)

    if "PYTEST_CURRENT_TEST" in os.environ:
        # use stdin for gracefully closing GUI when running
        # from pytest. This is required when measuring
        # coverage and process kill would cause measurement to be lost
        def listen_for_exit():
            input("Press any key to exit: ")
            dlg.Close()
            wx.Exit()

        input_thread = threading.Thread(target=listen_for_exit)
        input_thread.start()

        dlg.Show()
        app.MainLoop()
    else:
        dlg.ShowModal()

    return dlg



# SPDX-FileCopyrightText: 2025 adamws <adamws@users.noreply.github.com>
#
# SPDX-License-Identifier: GPL-3.0-or-later

import traceback

import wx

from .dialog_helper import MessageDialog
from .plugin_error import PluginError


class ErrorDialog(MessageDialog):
    def __init__(self, parent, e: Exception) -> None:
        super(ErrorDialog, self).__init__(parent, "kbplacer error")
        box = wx.BoxSizer(wx.VERTICAL)

        multiline_header = 0
        if type(e) == PluginError:
            message = e.message
            if "\n" in message:
                multiline_header = wx.TE_MULTILINE
            add_traceback = False
        else:
            message = getattr(e, "message", f"{e.__class__.__name__}: {e}")
            add_traceback = True

        error_header = wx.TextCtrl(
            self, value=message, style=wx.TE_READONLY | wx.NO_BORDER | multiline_header
        )
        error_header.SetBackgroundColour(self.GetBackgroundColour())
        self.adjust_size_to_text(error_header, message)

        error_icon = wx.ArtProvider.GetBitmap(
            wx.ART_ERROR, wx.ART_MESSAGE_BOX, wx.Size(32, 32)
        )
        icon = wx.StaticBitmap(self, bitmap=error_icon)

        icon_and_message_sizer = wx.BoxSizer(wx.HORIZONTAL)
        icon_and_message_sizer.Add(icon, 0, wx.ALL, 10)
        icon_and_message_sizer.Add(
            error_header, 0, wx.ALL | wx.ALIGN_CENTER_VERTICAL, 10
        )
        box.Add(icon_and_message_sizer, 0, wx.EXPAND | wx.ALL, 5)

        if add_traceback:
            traceback_str = traceback.format_exc()
            details_text = wx.TextCtrl(
                self,
                value=traceback_str,
                style=wx.TE_MULTILINE | wx.TE_READONLY | wx.HSCROLL,
            )
            self.adjust_size_to_text(details_text, traceback_str)
            box.Add(details_text, 0, wx.EXPAND | wx.ALL, 5)

        buttons = self.CreateButtonSizer(wx.OK)
        box.Add(buttons, 0, wx.EXPAND | wx.ALL, 5)

        self.SetSizerAndFit(box)


if __name__ == "__main__":
    import argparse

    from .dialog_helper import show_with_test_support

    parser = argparse.ArgumentParser(description="dialog test")
    parser.add_argument("--plugin-error", required=False, action="store_true", help="")
    args = parser.parse_args()

    if args.plugin_error:
        msg = "This is PluginError's message\nWhich can be multiline"
        e = PluginError(msg)
        show_with_test_support(ErrorDialog, None, e)
    else:
        try:
            _ = 1 / 0
        except Exception as e:
            show_with_test_support(ErrorDialog, None, e)
    print("exit ok")



# SPDX-FileCopyrightText: 2025 adamws <adamws@users.noreply.github.com>
#
# SPDX-License-Identifier: GPL-3.0-or-later

from typing import List, Optional

import wx

from .dialog_helper import MessageDialog


class WarningDialog(MessageDialog):
    def __init__(self, parent, warnings: List[str]) -> None:
        super(WarningDialog, self).__init__(parent, "kbplacer warning")
        warnings_count = len(warnings)
        if warnings_count == 0:
            msg = "WarningDialog must have at least one warning defined"
            raise RuntimeError(msg)
        box = wx.BoxSizer(wx.VERTICAL)

        header_text = f"Encountered {warnings_count} warnings"
        warnings_header = wx.TextCtrl(
            self, value=header_text, style=wx.TE_READONLY | wx.NO_BORDER
        )
        warnings_header.SetBackgroundColour(self.GetBackgroundColour())
        self.adjust_size_to_text(warnings_header, header_text)

        warning_icon = wx.ArtProvider.GetBitmap(
            wx.ART_WARNING, wx.ART_MESSAGE_BOX, wx.Size(32, 32)
        )
        icon = wx.StaticBitmap(self, bitmap=warning_icon)

        icon_and_message_sizer = wx.BoxSizer(wx.HORIZONTAL)
        icon_and_message_sizer.Add(icon, 0, wx.ALL, 10)
        icon_and_message_sizer.Add(
            warnings_header, 0, wx.ALL | wx.ALIGN_CENTER_VERTICAL, 10
        )
        box.Add(icon_and_message_sizer, 0, wx.EXPAND | wx.ALL, 5)

        warnings_str = "\n".join(warnings)
        details_text = wx.TextCtrl(
            self,
            value=warnings_str,
            style=wx.TE_MULTILINE | wx.TE_READONLY | wx.HSCROLL,
        )
        self.adjust_size_to_text(details_text, warnings_str)
        box.Add(details_text, 0, wx.EXPAND | wx.ALL, 5)

        buttons = self.CreateButtonSizer(wx.OK)
        box.Add(buttons, 0, wx.EXPAND | wx.ALL, 5)

        self.SetSizerAndFit(box)


def get_warnings_from_log(
    parent: wx.Window, log_file: str, *, prefix: str = "WARNING:"
) -> Optional[wx.Dialog]:
    # keeping python 3.8 compatibility which had no `removeprefix`:
    def _remove_prefix(text, prefix):
        if text.startswith(prefix):
            return text[len(prefix) :]
        return text

    with open(log_file, "r") as f:
        warnings = [
            _remove_prefix(l, prefix).strip() for l in f if l.startswith(prefix)
        ]
    if warnings:
        return WarningDialog(parent, warnings)
    return None


if __name__ == "__main__":
    import logging
    import os

    from .dialog_helper import show_with_test_support

    tmp_log = "warning_dialog_temp.log"
    logging.basicConfig(
        level=logging.DEBUG,
        filename=tmp_log,
        filemode="w",
        format="%(levelname)s: %(filename)s:%(lineno)d: %(message)s",
    )

    logger = logging.getLogger(__name__)
    logger.warning("This is warning number 1")
    logger.debug("This is debug which should be ignored")
    logger.warning("And this is warning number 2")

    show_with_test_support(get_warnings_from_log, None, tmp_log)
    os.remove(tmp_log)
    print("exit ok")



# SPDX-FileCopyrightText: 2025 adamws <adamws@users.noreply.github.com>
#
# SPDX-License-Identifier: GPL-3.0-or-later
import logging

logger = logging.getLogger(__name__)


class PluginError(Exception):
    def __init__(self, message: str) -> None:
        self.message = message
        logger.error(self.message.replace("\n", " "))
        super().__init__(self.message)



# SPDX-FileCopyrightText: 2025 adamws <adamws@users.noreply.github.com>
#
# SPDX-License-Identifier: GPL-3.0-or-later

from .element_position import ElementPosition, Side

DEFAULT_DIODE_POSITION = ElementPosition(5.08, 3.03, 90.0, Side.BACK)
ZERO_POSITION = ElementPosition(0, 0, 0, Side.FRONT)
